(function (d3) {
  'use strict';

  let data;
  let dimensions;
  let colorScale;

const parallelCoordinates = (selection, props) => {
  const {
    dimensions,
    data,
    margin,
    width,
    height
  } = props;

  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const yScales = {};
  dimensions.forEach(dim => {
    yScales[dim] = d3.scaleLinear()
      .domain(d3.extent(data, d => d[dim]))
      .range([innerHeight, 0]);
  });

  colorScale = d3.scaleOrdinal(d3.schemeCategory10)
    .domain(data.map(d => d['class']));
  
  const svg = selection.selectAll('svg').data([null]);
  const svgEnter = svg.enter().append('svg')
    .attr('width', width)
    .attr('height', height);

  const g = svgEnter.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);

  data.forEach(d => {
    const points = dimensions.map(dim => [yScales[dim](d[dim])]);
    const line = d3.line()
      .x((_, i) => i * (innerWidth / (dimensions.length - 1)))
      .y(d => d);

    g.append('path')
      .attr('class', 'line')
      .attr('d', line(points))
    	.style('stroke', colorScale(d['class']));
  });

  dimensions.forEach((dim, i) => {
    const yScale = yScales[dim];

    const yAxis = d3.axisLeft(yScale);
    g.append('g')
      .attr('class', 'axis')
      .attr('transform', `translate(${i * (innerWidth / (dimensions.length - 1))}, 0)`)
      .call(yAxis);

    g.append('text')
      .attr('class', 'axis-label')
      .attr('x', i * (innerWidth / (dimensions.length - 1)) + 5)
      .attr('y', -10)
      .text(dim);
    
    if (i === dimensions.length - 1) {
    text.attr('x', i * (innerWidth / (dimensions.length - 1)) - 5)
        .attr('text-anchor', 'end');
  }
  });
};


  const svg = d3.select('svg');

  const width = +svg.attr('width');
  const height = +svg.attr('height');

  d3.csv('https://gist.githubusercontent.com/yuting89830/75b3b407104edc96aa5d913d73232f1c/raw/08ac685d6709d23f4da82b514c25c713ccccac42/iris.csv')
    .then(loadedData => {
      data = loadedData;
      data.forEach(d => {
        d["petal length"] = +d["petal length"];
        d["petal width"] = +d["petal width"];
        d["sepal length"] = +d["sepal length"];
        d["sepal width"] = +d["sepal width"];
      });
      dimensions = ["sepal length", "sepal width", "petal length", "petal width"];
      const margin = { top: 30, right: 40, bottom: 30, left: 40 }; // Modify the margins as needed
      svg.call(parallelCoordinates, {
        dimensions,
        data,
        margin,
        width,
        height
      });
    // 添加圖例到圖表外部
      const legendGroup = d3.select('body').append('svg')
        .attr('class', 'legend-svg')
        .attr('width', 200) // 設置適當的寬度
        .attr('height', height)
        .append('g')
        .attr('transform', `translate(10, 20)`); // 調整圖例位置

      const legendItems = legendGroup.selectAll('.legend-item')
        .data(colorScale.domain())
        .enter().append('g')
        .attr('class', 'legend-item')
        .attr('transform', (d, i) => `translate(0, ${i * 20})`);

      legendItems.append('rect')
        .attr('x', 0)
        .attr('y', 0)
        .attr('width', 10)
        .attr('height', 10)
        .attr('fill', d => colorScale(d));

      legendItems.append('text')
        .attr('x', 15)
        .attr('y', 9)
        .text(d => d);

      const legendTitle = legendGroup.append('text')
        .attr('class', 'legend-title')
        .attr('x', 0)
        .attr('y', -10)
        .text('Legend');

      const legendSubtitle = legendGroup.append('text')
        .attr('class', 'legend-subtitle')
        .attr('x', 0)
        .attr('y', -30)
  